@@ -41,14 +41,9 @@
 #include "connect.h"
 #include "curlx/timeval.h"
 #include "socks.h"
-#include "multiif.h" /* for getsock macros */
 #include "curlx/inet_pton.h"
 #include "url.h"
 
-/* The last 2 #include files should be in this order */
-#include "curl_memory.h"
-#include "memdebug.h"
-
 #if defined(DEBUGBUILD) && !defined(CURL_DISABLE_VERBOSE_STRINGS)
 #define DEBUG_AND_VERBOSE
 #endif
@@ -137,7 +132,7 @@ CURLcode Curl_blockread_all(struct Curl_cfilter *cf,
 
   *pnread = 0;
   for(;;) {
-    timediff_t timeout_ms = Curl_timeleft(data, NULL, TRUE);
+    timediff_t timeout_ms = Curl_timeleft_ms(data, TRUE);
     if(timeout_ms < 0) {
       /* we already got the timeout */
       return CURLE_OPERATION_TIMEDOUT;
@@ -168,9 +163,9 @@ CURLcode Curl_blockread_all(struct Curl_cfilter *cf,
 
 #if defined(DEBUGBUILD) && !defined(CURL_DISABLE_VERBOSE_STRINGS)
 #define DEBUG_AND_VERBOSE
-#define sxstate(x,c,d,y) socksstate(x,c,d,y, __LINE__)
+#define sxstate(x, c, d, y) socksstate(x, c, d, y, __LINE__)
 #else
-#define sxstate(x,c,d,y) socksstate(x,c,d,y)
+#define sxstate(x, c, d, y) socksstate(x, c, d, y)
 #endif
 
 /* always use this function to change state, to make debugging easier */
@@ -210,9 +205,9 @@ static CURLproxycode socks_failed(struct socks_state *sx,
 }
 
 static CURLproxycode socks_flush(struct socks_state *sx,
-                                  struct Curl_cfilter *cf,
-                                  struct Curl_easy *data,
-                                  bool *done)
+                                 struct Curl_cfilter *cf,
+                                 struct Curl_easy *data,
+                                 bool *done)
 {
   CURLcode result;
   size_t nwritten;
@@ -308,7 +303,7 @@ static CURLproxycode socks4_req_add_user(struct socks_state *sx,
       return CURLPX_SEND_REQUEST;
   }
   else {
-    /* empty user name */
+    /* empty username */
     unsigned char b = 0;
     result = Curl_bufq_write(&sx->iobuf, &b, 1, &nwritten);
     if(result || (nwritten != 1))
@@ -336,8 +331,7 @@ static CURLproxycode socks4_resolving(struct socks_state *sx,
     result = Curl_resolv(data, sx->hostname, sx->remote_port,
                          cf->conn->ip_version, TRUE, &dns);
     if(result == CURLE_AGAIN) {
-      CURL_TRC_CF(data, cf, "SOCKS4 non-blocking resolve of %s",
-                  sx->hostname);
+      CURL_TRC_CF(data, cf, "SOCKS4 non-blocking resolve of %s", sx->hostname);
       return CURLPX_OK;
     }
     else if(result)
@@ -351,8 +345,7 @@ static CURLproxycode socks4_resolving(struct socks_state *sx,
   }
 
   if(result || !dns) {
-    failf(data, "Failed to resolve \"%s\" for SOCKS4 connect.",
-          sx->hostname);
+    failf(data, "Failed to resolve \"%s\" for SOCKS4 connect.", sx->hostname);
     return CURLPX_RESOLVE_HOST;
   }
 
@@ -469,16 +462,16 @@ static CURLproxycode socks4_check_resp(struct socks_state *sx,
 }
 
 /*
-* This function logs in to a SOCKS4 proxy and sends the specifics to the final
-* destination server.
-*
-* Reference :
-*   https://www.openssh.com/txt/socks4.protocol
-*
-* Note :
-*   Set protocol4a=true for  "SOCKS 4A (Simple Extension to SOCKS 4 Protocol)"
-*   Nonsupport "Identification Protocol (RFC1413)"
-*/
+ * This function logs in to a SOCKS4 proxy and sends the specifics to the final
+ * destination server.
+ *
+ * Reference :
+ *   https://www.openssh.com/txt/socks4.protocol
+ *
+ * Note :
+ *   Set protocol4a=true for  "SOCKS 4A (Simple Extension to SOCKS 4 Protocol)"
+ *   Nonsupport "Identification Protocol (RFC1413)"
+ */
 static CURLproxycode socks4_connect(struct Curl_cfilter *cf,
                                     struct socks_state *sx,
                                     struct Curl_easy *data)
@@ -737,7 +730,7 @@ static CURLproxycode socks5_auth_init(struct Curl_cfilter *cf,
     if(result || (nwritten != ulen))
       return CURLPX_SEND_REQUEST;
   }
-  buf[0] = (unsigned char) plen;
+  buf[0] = (unsigned char)plen;
   result = Curl_bufq_write(&sx->iobuf, buf, 1, &nwritten);
   if(result || (nwritten != 1))
     return CURLPX_SEND_REQUEST;
@@ -817,7 +810,7 @@ static CURLproxycode socks5_req1_init(struct socks_state *sx,
     const size_t hostname_len = strlen(sx->hostname);
     desttype = 3;
     destination = (const unsigned char *)sx->hostname;
-    destlen = (unsigned char) hostname_len; /* one byte length */
+    destlen = (unsigned char)hostname_len; /* one byte length */
   }
 
   req[3] = desttype;
@@ -864,8 +857,7 @@ static CURLproxycode socks5_resolving(struct socks_state *sx,
     result = Curl_resolv(data, sx->hostname, sx->remote_port,
                          cf->conn->ip_version, TRUE, &dns);
     if(result == CURLE_AGAIN) {
-      CURL_TRC_CF(data, cf, "SOCKS5 non-blocking resolve of %s",
-                  sx->hostname);
+      CURL_TRC_CF(data, cf, "SOCKS5 non-blocking resolve of %s", sx->hostname);
       return CURLPX_OK;
     }
     else if(result)
@@ -879,8 +871,7 @@ static CURLproxycode socks5_resolving(struct socks_state *sx,
   }
 
   if(result || !dns) {
-    failf(data, "Failed to resolve \"%s\" for SOCKS5 connect.",
-          sx->hostname);
+    failf(data, "Failed to resolve \"%s\" for SOCKS5 connect.", sx->hostname);
     presult = CURLPX_RESOLVE_HOST;
     goto out;
   }
@@ -897,8 +888,7 @@ static CURLproxycode socks5_resolving(struct socks_state *sx,
   }
 #endif
   if(!hp) {
-    failf(data, "Failed to resolve \"%s\" for SOCKS5 connect.",
-          sx->hostname);
+    failf(data, "Failed to resolve \"%s\" for SOCKS5 connect.", sx->hostname);
     presult = CURLPX_RESOLVE_HOST;
     goto out;
   }
@@ -1215,7 +1205,7 @@ static void socks_proxy_cf_free(struct Curl_cfilter *cf)
   struct socks_state *sxstate = cf->ctx;
   if(sxstate) {
     Curl_bufq_free(&sxstate->iobuf);
-    free(sxstate);
+    curlx_free(sxstate);
     cf->ctx = NULL;
   }
 }
@@ -1247,7 +1237,7 @@ static CURLcode socks_proxy_cf_connect(struct Curl_cfilter *cf,
     return result;
 
   if(!sx) {
-    cf->ctx = sx = calloc(1, sizeof(*sx));
+    cf->ctx = sx = curlx_calloc(1, sizeof(*sx));
     if(!sx) {
       result = CURLE_OUT_OF_MEMORY;
       goto out;
@@ -1398,7 +1388,7 @@ static CURLcode socks_cf_query(struct Curl_cfilter *cf,
 
 struct Curl_cftype Curl_cft_socks_proxy = {
   "SOCKS",
-  CF_TYPE_IP_CONNECT|CF_TYPE_PROXY,
+  CF_TYPE_IP_CONNECT | CF_TYPE_PROXY,
   0,
   socks_proxy_cf_destroy,
   socks_proxy_cf_connect,
